@@ -25,11 +25,9 @@ except ImportError:
     pass
 
 from xml2rfc import strings, log
-from xml2rfc.uniscripts import is_script
 from xml2rfc.util.date import extract_date, augment_date, format_date, get_expiry_date
 from xml2rfc.util.name import short_author_ascii_name_parts, full_author_name_expansion, short_author_name_parts
-from xml2rfc.util.unicode import ( punctuation, unicode_replacements, unicode_content_tags, bare_unicode_tags,
-    bare_latin_tags, unicode_attributes, downcode, downcode_punctuation)
+from xml2rfc.util.unicode import is_svg
 from xml2rfc.utils import namespaces, find_duplicate_ids, slugify
 
 
@@ -55,7 +53,7 @@ default_options.__dict__ = {
         'css': None,
         'config_file': None,
         'country_help': False,
-        'date': datetime.date.today(),
+        'date': None,
         'datestring': None,
         'debug': False,
         'docfile': False,
@@ -100,6 +98,7 @@ default_options.__dict__ = {
         'raw': False,
         'rfc': None,
         'rfc_base_url': 'https://www.rfc-editor.org/rfc/',
+        'rfc_local': True,
         'rfc_reference_base_url': 'https://rfc-editor.org/rfc/',
         'silence': default_silenced_messages,
         'skip_config_files': False,
@@ -118,6 +117,7 @@ default_options.__dict__ = {
         'v3': True,
         'vocabulary': 'v2',
         'widows': 2,
+        'warn_bare_unicode': False,
     }
 
 class _RfcItem:
@@ -442,11 +442,11 @@ class BaseRfcWriter:
 
     # -------------------------------------------------------------------------
 
-    def __init__(self, xmlrfc, quiet=None, options=default_options, date=datetime.date.today()):
+    def __init__(self, xmlrfc, quiet=None, options=default_options, date=None):
         if not quiet is None:
             options.quiet = quiet
         self.options = options
-        self.date = date
+        self.date = date if date is not None else datetime.date.today()
         self.expire_string = ''
         self.ascii = False
         self.nbws_cond = u'\u00A0'
@@ -609,7 +609,7 @@ class BaseRfcWriter:
                 # period
                 initials = initials.strip()
                 initials += '.' if not initials.endswith('.') else ''
-                initials = re.sub('([^.]) ', '\g<1>. ', initials)
+                initials = re.sub(r'([^.]) ', r'\g<1>. ', initials)
             else:
                 initials = initials_list[0] + "."
         return initials
@@ -1347,7 +1347,7 @@ class BaseRfcWriter:
                 xml2rfc.log.warn("No (or empty) 'docName' attribute in the <rfc/> element -- can't insert draft name on first page.")
             if docName and '.' in docName:
                 xml2rfc.log.warn("The 'docName' attribute of the <rfc/> element should not contain any filename extension: docName=\"draft-foo-bar-02\".")
-            if docName and not rfcnum and not re.search('-\d\d$', docName):
+            if docName and not rfcnum and not re.search(r'-\d\d$', docName):
                 xml2rfc.log.warn("The 'docName' attribute of the <rfc/> element should have a revision number as the last component: docName=\"draft-foo-bar-02\".")
             self.write_title(title.text, docName, title.sourceline)
 
@@ -1615,7 +1615,7 @@ deprecated_element_tags = set([
 ])
 
 deprecated_attributes = [
-    # element, attrbute
+    # element, attribute
     ('artwork', 'height'),
     ('artwork', 'width'),
     ('artwork', 'xml:space'),
@@ -1717,13 +1717,13 @@ xref_tags   = get_xref_tags()
 
 class BaseV3Writer(object):
 
-    def __init__(self, xmlrfc, quiet=None, options=default_options, date=datetime.date.today()):
+    def __init__(self, xmlrfc, quiet=None, options=default_options, date=None):
         global v3_rnc_file, v3_rng_file, v3_schema
         self.xmlrfc = xmlrfc
         self.tree = xmlrfc.tree if xmlrfc else None
         self.root = self.tree.getroot() if xmlrfc else None
         self.options = options
-        self.date = date
+        self.date = date if date is not None else datetime.date.today()
         self.v3_rnc_file = v3_rnc_file
         self.v3_rng_file = v3_rng_file
         self.v3_rng = lxml.etree.RelaxNG(file=self.v3_rng_file)
@@ -2014,6 +2014,7 @@ class BaseV3Writer(object):
         authors = self.root.findall('front/author')
         authors = [ a for a in authors if a.get('role') != 'contributor' ]
         surnames = [ n for n in [ short_author_ascii_name_parts(a)[1] for a in authors ] if n ]
+        text = ''
         if len(surnames) == 1:
             text = surnames[0]
         elif len(surnames) == 2:
@@ -2033,59 +2034,6 @@ class BaseV3Writer(object):
             text = 'Expires %s' % format_date(*parts, legacy=self.options.legacy_date_format)
         return text
 
-    def downcode_punctuation(self):
-        self.downcode(replacements=punctuation)
-        self.downcode_reference_punctuation()
-
-    def downcode(self, replacements=unicode_replacements):
-        """
-
-        Traverses an lxml.etree and replaces unicode characters with the proper
-        equivalents specified in rfc2629-xhtml.ent, resulting in no non-ascii
-        characters except in elements that explicitly permit non-ascii content.
-
-        """
-        for e in self.tree.iter():
-            if e.text:
-                if not e.tag in unicode_content_tags:
-                    try:
-                        e.text.encode('ascii')
-                    except UnicodeEncodeError:
-                        e.text = downcode(e.text, replacements=replacements)
-                elif e.tag in bare_unicode_tags:
-                    pass
-                elif e.tag in bare_latin_tags and is_script(e.text, 'Latin'):
-                    pass
-                elif not e.get('ascii'):
-                    try:
-                        e.text.encode('ascii')
-                    except UnicodeEncodeError:
-                        e.text = downcode(e.text, replacements=replacements)
-            if e.tail:
-                if not e.getparent().tag in unicode_content_tags:
-                    try:
-                        e.tail.encode('ascii')
-                    except UnicodeEncodeError:
-                        e.tail = downcode(e.tail, replacements=replacements)
-
-    def downcode_attributes(self, replacements=unicode_replacements):
-        for e in self.tree.iter():
-            for key in e.attrib.keys():
-                if not (e.tag, key) in unicode_attributes:
-                    try:
-                        e.get(key).encode('ascii')
-                    except UnicodeEncodeError:
-                        e.set(key, downcode(e.get(key), replacements=replacements))
-
-    def downcode_reference_punctuation(self):
-        for r in self.tree.xpath('.//references'):
-            for e in r.iter():
-                for key in e.attrib.keys():
-                    try:
-                        e.get(key).encode('ascii')
-                    except UnicodeEncodeError:
-                        e.set(key, downcode_punctuation(e.get(key)))
-
     def pretty_print_prep(self, e, p):
         ind = self.options.indent
         ## The actual printing is done in self.write()
@@ -2120,18 +2068,17 @@ class BaseV3Writer(object):
                 else:
                     if e.text and re.search(r'\n[ \t]+$', e.text):
                         e.text = re.sub(r'\n[ \t]+$', '\n'+' '*(i), e.text)
-                    
-            #
+
             for c in e:
                 indent(c, i+ind)
-            #
-            if e.tag not in all_inline_tags:# and e.tag not in ['artwork', 'sourcecode', ]:
+
+            if e.tag not in all_inline_tags:
                 if e.tail is None or e.tail.strip()=='':
                     if e.getnext() != None:
                         e.tail = '\n'+' '*i
                     else:
                         e.tail = '\n'+' '*(i-ind)
-                else:
+                elif not is_svg(e):
                     self.warn(e, 'Unexpected text content after block tag: <%s>%s' % (e.tag, e.tail))
             else:
                 if e.tail != None and e.tail.strip()=='' and '\n' in e.tail:
@@ -2169,7 +2116,9 @@ class BaseV3Writer(object):
             self.warn(e, 'Duplicate xsd:ID attribute %s="%s" found.  This will cause validation failure.' % (attr, id, ))
 
         try:
-            self.v3_rng.assertValid(self.tree)
+            # Use a deepcopy to avoid any memory issues.
+            tree = copy.deepcopy(self.tree)
+            self.v3_rng.assertValid(tree)
             return True
         except Exception as e:
             lxmlver = lxml.etree.LXML_VERSION[:3]
@@ -2213,18 +2162,14 @@ class BaseV3Writer(object):
     def validate_after(self, e, p):
         # XXX: There is an issue with exponential increase in validation time
         # as a function of the number of attributes on the root element, on
-        # the order of minutes for the full set of possble attribute values.
+        # the order of minutes for the full set of possible attribute values.
         # See https://bugzilla.gnome.org/show_bug.cgi?id=133736 .  In our
         # schema, there is no dependency in the underlying schema on the root
         # element attributes.  In order to avoid very long validation times, we
         # strip the root attributes before validation, and put them back
-        # afterwards.  
-        for k in e.keys():
-            if not e.get(k):
-                del e.attrib[k]
-        attrib = copy.deepcopy(e.attrib) 
-        for k in attrib.keys():
-            del e.attrib[k]
+        # afterwards.
+        attrib = copy.deepcopy(e.attrib)
+        e.attrib.clear()
         #
         if not self.validate('after', warn=True):
             self.note(None, "Schema validation failed for input document")
@@ -2233,8 +2178,7 @@ class BaseV3Writer(object):
         #
         keys = list(attrib.keys())
         keys.sort()
-        for k in keys:
-            e.set(k, attrib[k])
+        e.attrib.update(attrib)
 
     def remove(self, p, e):
         # Element.remove(child) removes both the child and its tail, so in
